# Copyright (C) 2018 Mo Zhou <cdluminate@gmail.com>
import re
import sys
import argparse


def colorize(msg: str):
    msg = re.sub('<replaceable>', '\x1b[0;33m', msg)
    msg = re.sub('</replaceable>', '\x1b[;m', msg)
    msg = re.sub('<literal>', '\x1b[0;36m', msg)
    msg = re.sub('</literal>', '\x1b[;m', msg)
    msg = re.sub('<optional>', '\x1b[1;33m[\x1b[;m', msg)
    msg = re.sub('</optional>', '\x1b[1;33m]\x1b[;m', msg)
    msg = re.sub('<filename>', '\x1b[1;35m', msg)
    msg = re.sub('</filename>', '\x1b[;m', msg)
    msg = re.sub('<command>', '\x1b[1;32m', msg)
    msg = re.sub('</command>', '\x1b[;m', msg)
    msg = re.sub('<ulink (.*)>(.*)</ulink>', '\\2 [\\1]', msg)
    msg = re.sub('<emphasis>', '\x1b[0;31m', msg)
    msg = re.sub('</emphasis>', '\x1b[;m', msg)
    return msg


if __name__ == '__main__':
    ag = argparse.ArgumentParser()
    ag.add_argument('src', type=str)
    ag = ag.parse_args()

    lines = open(ag.src, 'r') .readlines()
    msgid = []
    msgstr = []
    cursor = None
    for line in lines:
        if re.match('^$', line): continue
        if re.match('^#~.*', line): continue
        if re.match('^#.*', line): continue

        if re.match('^msgid.*', line):
            msgid.append('')
            cursor = msgid
            cursor[-1] += eval(line.replace('msgid', ''))
        elif re.match('^msgstr.*', line):
            msgstr.append('')
            cursor = msgstr
            cursor[-1] += eval(line.replace('msgstr', ''))
        else:
            cursor[-1] += eval(line)
    messages = list(reversed(msgstr))
    try:
        for i in range(0, len(messages), 2):
            cmd = colorize(messages[i+1])
            note = colorize(messages[i])

            #print('cmd', cmd)
            #print('note', note)

            padding = '\x1b[1;36m  │\x1b[;m'
            print(padding)
            print(padding, cmd)
            print(padding)
            print(padding, note)
            print(padding)
            print()
            print('                                                             -- Debian 参考卡片')
            print('%')
    except IndexError as e:
        pass



'''
https://www.debian.org/doc/manuals/debian-reference/debian-reference.en.txt
'''
__URL__ = 'https://www.debian.org/doc/manuals/debian-reference/debian-reference.zh-cn.txt'
import re
import os
from subprocess import call

White   = lambda x: '\x1b[37;1m' + x + '\x1b[;m'
Green   = lambda x: '\x1b[32;1m' + x + '\x1b[;m'
Red     = lambda x: '\x1b[31;1m' + x + '\x1b[;m'
Cyan    = lambda x: '\x1b[36;1m' + x + '\x1b[;m'
cyan    = lambda x: '\x1b[36m' + x + '\x1b[;m'
Yellow  = lambda x: '\x1b[33;1m' + x + '\x1b[;m'
yellow  = lambda x: '\x1b[33m' + x + '\x1b[;m'
violet  = lambda x: '\x1b[35m' + x + '\x1b[;m'

def colorize(line):
    if re.match('^\s*\$.*', line):
        return Green(line)
    elif re.match('^\s*#.*', line):
        return Red(line)
    else:
        line = re.sub('(“.*?”)', '\x1b[35;1m\\1\x1b[;m', line)
        line = re.sub('(".*?")', '\x1b[35;1m\\1\x1b[;m', line)
        line = re.sub('(\'.*?\')', '\x1b[35;1m\\1\x1b[;m', line)
        #line = re.sub('^(\s*[\d\.]*)', '\x1b[33;1m\\1\x1b[;m', line)
        line = re.sub('([\w\-]+?\(\d\))', '\x1b[34;1m\\1\x1b[m', line)
        line = re.sub('(^\s*\*)', '\x1b[33;1m\\1\x1b[;m', line)
        line = re.sub('(^\s*\+)', '\x1b[33;1m\\1\x1b[;m', line)
        if re.match(r'\s*注意\s*', line): line = re.sub('注意', '\x1b[33;1m注意\x1b[m', line)
        if re.match(r'\s*小心\s*', line): line = re.sub('小心', '\x1b[33;1m小心\x1b[m', line)
        if re.match(r'\s*提示\s*', line): line = re.sub('提示', '\x1b[36;1m提示\x1b[m', line)
        if re.match(r'\s*警告\s*', line): line = re.sub('警告', '\x1b[31;1m警告\x1b[m', line)
        #line = re.sub('(\d*?)', '\x1b[36m\\1\x1b[m', line)
        return line


if __name__ == '__main__':

    if not os.path.exists('debian-reference.zh-cn.txt'):
        call(['wget', __URL__, '-O', 'debian-reference.zh-cn.txt'])

    lines = open('debian-reference.zh-cn.txt', 'r').readlines()
    for i, line in enumerate(lines):
        line = line.rstrip()
        # deal with tables
        if re.match(r'\s*\+\-*\+\s*', line) and not lines[i+1].strip():
            line = re.sub(r'\+\-', '└-', line)
            line = re.sub(r'\-\+', '-┘', line)
            line = re.sub(r'\-', '─', line)
        elif re.match(r'\s*\+\-*\+\s*', line) and not lines[i-1].strip():
            line = re.sub(r'\+\-', '┌-', line)
            line = re.sub(r'\-\+', '-┐', line)
            line = re.sub(r'\-', '─', line)
        elif re.match(r'\s*\|[\-\+]*\|\s*', line):
            line = re.sub(r'\|\-', '├-', line)
            line = re.sub(r'\-\|', '-┤', line)
            line = re.sub(r'\-\+\-', '-┼-', line)
            line = re.sub(r'\-', '─', line)
        elif re.match(r'\s*\|.*\|*\|\s*', line):
            line = re.sub(r'\|', '│', line)

        # colorization
        if re.match(r'^\S+.*', line) and i > 0:
            print('                           -- Osamu Aoki (青木修), Debian 参考手册（版本 2.73）')
            print('%')
            print(White(line))
        else:
            line = colorize(line)
            print(line)



#!/usr/bin/python3
'''
Convert "cookie" files into "dat" files for fortune.
This script requires python3 >= 3.6
Copyright (C) 2018 Mo Zhou <cdluminate@gmail.com>
'''
from typing import *
import glob
import os


def cookie2dat(src: str) -> None:
    '''
    Read "cookie" and do preprocessing. :src: refers to source file path.
    '''
    dest = src.replace('.cookie', '.dat')
    with open(src, 'r') as fsrc, open(dest, 'w') as fdest:
        contents = fsrc.readlines()
        while contents:
            if contents[0].startswith('#'):
                contents.pop(0)
            else:
                break
        if contents:
            fdest.writelines(contents)
            print(f'Converted {src} -> {dest}')
        else:
            print(f'{src} is empty. skipped')


if __name__ == '__main__':

    cookies = glob.glob('./**/**.cookie', recursive=True)
    print(f'{__file__}: Found {len(cookies)} cookie files')
    for cookie in cookies:
        cookie2dat(cookie)
